/*********************************************************************/
/* tclparse V2.00 from 14.March 1997                                 */
/* Copyright 1996/97 Stephan Weiss, Heiko Grossmann                  */
/*                   University of Stuttgart, Germany                */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/* routines for stack managing                                       */
/*                                                                   */
/*********************************************************************/

#include "../include/stack.h"
#include "../include/parsing.h"
#include "../include/const.h"
#include <string.h>



#include <iostream.h>



CStack::CStack( void )
{
  root = new SElement;
  root->prev = root;
  root->next = root;

  internalPtr = root->next;
}


CStack::~CStack( void )
{
  //Ein Zeiger definieren, der die gesamte Liste durchlaeuft.
  SElement *dummy;

  //Den Zeiger auf das erste Element setzen.
  dummy = root->next;

  //Die Liste solange durchlaufen, bis der Zeiger wieder auf der Wurzel steht.
  //Existieren keine Elemente, wird die Schleife nicht durchlaufen.
  while( dummy != root )
    {
      //Den Zeiger auf das naechste Element setzen.
      dummy = dummy->next;

      //Das vorherige Element loeschen.
      delete dummy->prev;
    }

  //Zum Schluss noch den Wurzelzeiger loeschen.
  delete root;
}


char *CStack::First( void )
{
  if( root->next == root )
    return NULL;

  internalPtr = root->next;
  return internalPtr->content;
}


char *CStack::Next( void )
{
  if( internalPtr->next == root )
    return NULL;

  internalPtr = internalPtr->next;
  return internalPtr->content;
}


char *CStack::Concat( char *separator)
{
  char result[ bufferSize ] = "";

  //Einen Hilfszeiger definieren und ihn auf das erste Element setzen.
  SElement *dummy;
  dummy = root->next;

  //Alle Elemente ausser dem Wurzelelement anhaengen.
  while( dummy != root )
    {
      strcat( result, dummy->content );

      dummy = dummy->next;
      if( dummy == root )
	break;

      strcat( result, separator );
    }

  if( !strcmp( result, "" ) )
    strcpy( result, " " );

  CopyToBuffer( result );
  return Buffer();
}


int CStack::Contains( char *string )
{
  //Einen Hilfszeiger definieren und ihn auf das erste Element setzen.
  SElement *dummy;
  dummy = root->next;

  //Alle Elemente ausser dem Wurzelelement testen.
  while( dummy != root )
    {
      //Den Vergleich durchfuehren.
      if( strcmp( dummy->content, string ) == 0 )
	return 1;
      else
	dummy = dummy->next;
    }

  //Kommt die Programmausfuehrung bis hier, so wurde das Element nicht gefunden.
  return 0;
}


char *CStack::Pop( void )
{
  //Ist der Stack leer, kann auch nichts entfernt werden.
  if( root->prev == root )
    return NULL;

  //Einen Hilfszeiger auf das zu entfernende Element setzen.
  SElement *dummy = root->prev;

  //Einen Hilfszeiger auf das neue oberste Element setzen.
  SElement *newLast = dummy->prev;

  //Die notwendigen Verknuepfungen durchfuehren.
  newLast->next = root;
  root->prev = newLast;

  CopyToBuffer( dummy->content );

  delete dummy;

  return Buffer();
}


void CStack::Push( char *string )
{
  //Ein neues Element erzeugen.
  SElement *newElement = new SElement;

  //Das neue Element initialisieren.
  newElement->content = new char[ strlen( string )+1 ];
  strcpy( newElement->content, string );

  //Einen Hilfszeiger definieren und ihn auf das letzte Element setzen.
  SElement *dummy;
  dummy = root->prev;

  //Das Element wird hinten angehaengt.
  dummy->next = newElement;
  root->prev = newElement;
  newElement->prev = dummy;
  newElement->next = root;
}


char *CStack::Top( void )
{
  //Ist der Stack leer, kann auch nichts zurueckgeliefert werden.
  if( root->prev == root )
    {
      CopyToBuffer( "\0" );
      return Buffer();
    }

  CopyToBuffer( root->prev->content );
  return Buffer();
}


CStack &CStack::operator=( CStack &stack )
{
  if( this == &stack )
    return *this;

  //Den Stack vollkommen loeschen.
  while( root->next != root )
    Pop();

  //Dann alle auf den Stack bringen.
  char *dummy = stack.First();
  while( dummy != NULL )
    {
      Push( dummy );
      dummy = stack.Next();
    }

  return *this;
}


//######################################################################


CTclStack::CTclStack( void )
{
}


CTclStack::~CTclStack( void )
{
}


char *CTclStack::Concat( stackType id, char* separator)
{
  switch( id )
    {
    case parens:
      return parenStack.Concat( separator );

    case braces:
      return braceStack.Concat( separator );

    case quotes:
      return quoteStack.Concat( separator );

    case brackets:
      return bracketStack.Concat( separator );

    case procedures:
      return procedureStack.Concat( separator );

    case commands:
      return commandStack.Concat( separator );

    case normal:
      return normalStack.Concat( separator );

    default:
      CopyToBuffer( "\0" );
      return Buffer();
    }
}


int CTclStack::Contains( stackType id, char *string )
{
  switch( id )
    {
    case parens:
      return parenStack.Contains( string );

    case braces:
      return braceStack.Contains( string );

    case quotes:
      return quoteStack.Contains( string );

    case brackets:
      return bracketStack.Contains( string );

    case procedures:
      return procedureStack.Contains( string );

    case commands:
      return commandStack.Contains( string );

    case normal:
      return normalStack.Contains( string );

    default:
      return 0;
    }
}


char *CTclStack::Pop( stackType id )
{
  switch( id )
    {
    case parens:
      return parenStack.Pop();

    case braces:
      return braceStack.Pop();

    case quotes:
      return quoteStack.Pop();

    case brackets:
      return bracketStack.Pop();

    case procedures:
      return procedureStack.Pop();

    case commands:
      return commandStack.Pop();

    case normal:
      return normalStack.Pop();

    default:
      CopyToBuffer( "\0" );
      return Buffer();
    }
}


void CTclStack::Push( stackType id, char *string)
{
  switch( id )
    {
    case parens:
      parenStack.Push( string );
      return;

    case braces:
      braceStack.Push( string );
      return;

    case quotes:
      quoteStack.Push( string );
      return;

    case brackets:
      bracketStack.Push( string );
      return;

    case procedures:
      procedureStack.Push( string );
      return;

    case commands:
      commandStack.Push( string );
      return;

    case normal:
      normalStack.Push( string );
      return;

    default:
      return;
    }
}


char *CTclStack::Top( stackType id )
{
  switch( id )
    {
    case parens:
      return parenStack.Top();

    case braces:
      return braceStack.Top();

    case quotes:
      return quoteStack.Top();

    case brackets:
      return bracketStack.Top();

    case procedures:
      return procedureStack.Top();

    case commands:
      return commandStack.Top();

    case normal:
      return normalStack.Top();

    default:
      CopyToBuffer( "\0" );
      return Buffer();
    }
}


CTclStack &CTclStack::operator=( CTclStack &stack )
{
  if( this == &stack )
    return *this;

  //Alle einzelnen Stacks einander zuweisen.
  parenStack = stack.parenStack;
  braceStack = stack.braceStack;
  quoteStack = stack.quoteStack;
  bracketStack = stack.bracketStack;
  procedureStack = stack.procedureStack;
  commandStack = stack.commandStack;
  normalStack = stack.normalStack;

  return *this;
}
